import os
import json
from .base import PluginVue, PluginUniapp


class PluginViteConfig(PluginVue, PluginUniapp):
    vite_config_standard_name = 'vite.config.js'
    dek_config_marker = '//@marker-dek-config-add'

    @property
    def all_vite_imports(self):
        result = []

        for data_dek in self.dek_info_list:
            items = (data_dek.get(self.vite_config_standard_name) or {}).get('imports') or []
            result.extend(items)

        return result

    @property
    def all_vite_plugins(self):
        result = []

        for data_dek in self.dek_info_list:
            items = (data_dek.get(self.vite_config_standard_name) or {}).get('plugins') or []
            result.extend(items)

        return result

    @property
    def all_vite_alias(self):
        result = {}

        for data_dek in self.dek_info_list:
            items = (data_dek.get(self.vite_config_standard_name) or {}).get('alias') or {}
            result.update(items)

        return result

    @property
    def all_vite_buildinputs(self):
        result = {}

        for data_dek in self.dek_info_list:
            items = (data_dek.get(self.vite_config_standard_name) or {}).get('buildInputs') or {}
            result.update(items)

        return result

    @property
    def all_vite_scssheader(self):
        result = []

        for data_dek in self.dek_info_list:
            items = (data_dek.get(self.vite_config_standard_name) or {}).get('scssHeader') or []
            result.extend(items)

        return result

    def run(self):
        if self.is_uniapp_project():
            if self.get_uniapp_vue_version() != 3:
                return
        else:
            version = self.get_vue_version()
            if not version or version[0] != 3:
                return
        if not os.path.isfile(self.vue_config_standard_filepath):
            return
        s = self.load_text(self.vue_config_standard_filepath)
        str_list = []

        for item in sorted(set(self.all_vite_plugins)):
            mk = f'marker--addPlugin--{self.get_data_uid(item)}'
            if mk not in s:
                str_list.append(f'dekConfig.addPlugin({json.dumps(mk)}, {item})')

        for key in sorted(self.all_vite_alias):
            value = self.all_vite_alias[key]
            mk = f'marker--addAlias--{self.get_data_uid([key, value])}'
            if mk not in s:
                str_list.append(f'dekConfig.addAlias({json.dumps(mk)}, {json.dumps(key)}, {value})')

        for item in sorted(set(self.all_vite_scssheader)):
            mk = f'marker--addScssHeader--{self.get_data_uid(item)}'
            if mk not in s:
                str_list.append(f'dekConfig.addScssHeader({json.dumps(mk)}, {json.dumps(item)})')

        for key in sorted(self.all_vite_buildinputs):
            value = self.all_vite_buildinputs[key]
            mk = f'marker--addBuildInputs--{self.get_data_uid([key, value])}'
            if mk not in s:
                str_list.append(f'dekConfig.addBuildInputs({json.dumps(mk)}, {json.dumps(key)}, {value})')

        index = s.find(self.dek_config_marker)
        ss = ''.join([
            s[:index], '\n',
            '\n'.join([x for x in self.all_vite_imports if x not in s]), '\n',
            '\n'.join(str_list), '\n',
            s[index:]
        ])
        self.save_text(self.vue_config_standard_filepath, ss)

    @property
    def vue_config_standard_filepath(self):
        return os.path.join(self.project_dir, self.vite_config_standard_name)



import svgpathtools.path

COMMAND_RE_LENGTH = {
    'A': 7
}


def array_fix(array, command):
    result = []
    for x in array:
        if x.startswith('01'):
            result.extend(['0', '1'])
            if x[2:]:
                result.append(x[2:])
        elif x.startswith('00'):
            result.extend(['0', '0'])
            if x[2:]:
                result.append(x[2:])
        elif x[0] == '0' and not x[1:].startswith('.'):
            result.append(x[0])
            if x[1:]:
                result.append(x[1:])
        else:
            result.append(x)
    length = COMMAND_RE_LENGTH.get(command and command.upper())
    if length is not None and len(result) > length:
        result = result[:length]
    return result


def _tokenize_path(self, pathdef):
    result = []
    command_last = None
    for x in svgpathtools.path.COMMAND_RE.split(pathdef):
        if x in svgpathtools.path.COMMANDS:
            command_last = x
            result.append(x)
        result.extend(array_fix(svgpathtools.path.FLOAT_RE.findall(x), command_last))
    return result


svgpathtools.path.Path._tokenize_path = _tokenize_path



import os
import tempfile
from dektools.file.operation import write_file, remove_path
from dekgen.tmpl.template import Template
from .base import Plugin


class PluginOverrideRootTemplate(Template):
    file_ignore_name = 'dek-override'


class PluginOverrideRoot(Plugin):
    template_cls = PluginOverrideRootTemplate

    override_key = 'override'
    dek_overrides_dir_name = 'dek-override'

    def run(self):
        ignore_info = self.merge_from_key(self.override_key)
        dir_temp = tempfile.mkdtemp()
        for dir_dek in self.dek_dir_list:
            src = os.path.join(dir_dek, self.dek_overrides_dir_name)
            if os.path.exists(src):
                write_file(dir_temp, ma=src)
        for filename, info in ignore_info.items():
            write_file(
                os.path.join(dir_temp, self.template_cls.get_file_ignore(filename)),
                s="\n".join([x for x, b in info.items() if b])
            )
        self.template_full.render_dir(self.project_dir, dir_temp, force_close_tpl=True)
        remove_path(dir_temp)



import os
import shutil
from .base import PluginVue


class PluginPackages(PluginVue):
    path_generated_package = '@'
    excludes = [
        "dek-override",
        "LICENSE",
        "package.dek.json",
        "package.json",
        "README.md"
    ]

    def run(self):
        version = self.get_vue_version()
        if not version or version[0] != 3:
            return
        package_map = {}
        path_root = self.get_dek_path_generated(self.path_generated_package)
        if os.path.exists(path_root):
            shutil.rmtree(path_root)
        for dir_dek in self.dek_dir_list:
            if dir_dek in self.dek_dev_dir_list:
                continue
            package_map[self.get_package_json(dir_dek)['name']] = dir_dek
            if dir_dek == self.project_dir:
                continue
            pkg_path = dir_dek[len(self.node_modules_dir) + 1:]
            for file in os.listdir(dir_dek):
                if file in self.excludes:
                    continue
                filepath = os.path.join(dir_dek, file)
                p = self.get_dek_path_generated(self.path_generated_package, pkg_path)
                if os.path.isfile(filepath):
                    if not os.path.exists(p):
                        os.makedirs(p)
                    shutil.copyfile(filepath, os.path.join(p, file))
                else:
                    shutil.copytree(filepath, os.path.join(p, file))
        self.share_data['packages'] = package_map



import os
import glob
from .fix import svgpathtools
from svgpathtools import svg2paths2
from .base import Plugin


def get_svg_data(filepath):
    paths, attributes, svg_attributes = svg2paths2(filepath)
    vb = [int(x) for x in svg_attributes['viewBox'].split(' ') if x]
    p_list = []
    result = {
        "w": vb[2],
        "h": vb[3],
        "p": p_list
    }
    for i in range(len(paths)):
        attr = attributes[i]
        d = attr.get('d')
        if not d:
            d = paths[i].d()
        fill = attr.get('fill')
        pd = dict(d=d)
        if fill and fill != 'currentColor':
            pd['c'] = fill
        p_list.append(pd)
    return result


class PluginDv3Svg(Plugin):
    path_generated_icons = 'dv3/icons'
    glob_icon_svg = '/**/dv3/icons/*.svg'

    def run(self):
        for filepath in self.list_glob_filepath(self.glob_icon_svg):
            name = os.path.splitext(os.path.basename(filepath))[0]
            self.save_text(self.get_dek_path_generated(self.path_generated_icons, f'{name}.svg'),
                           self.load_text(filepath))
            self.save_json(
                self.get_dek_path_generated(self.path_generated_icons, f'{name}.json'),
                get_svg_data(filepath))



from dektools.dict import assign
from .base import Plugin


class PluginPackageJson(Plugin):
    def run(self):
        for data in self.dek_info_list:
            package_json = data.get(self.package_standard_name) or {}
            if package_json:
                package_json = self.render_json(package_json)
                d = self.load_package_standard()
                d = assign(d, package_json)
                d.pop('dek', None)
                self.save_json(self.package_standard_filepath, d)



from .uniapp_vars import *
from .override_root import *
from .package_json import *
from .babel_config import *
from .vue_config import *
from .vite_config import *
from .favicon_ico import *
from .dv3 import *
from .dv3_svg import *
from .dv3_server import *
from .dv3_yaml import *
from .dv3_env_vars import *
from .packages import *
from .requests import *



import os
import json
import hashlib
from .base import Plugin
from dektools.file import write_file
from dekgen.resloader import ResLoader


class PluginDv3Yaml(Plugin):
    yaml_prefix = '.yaml'
    yaml_suffix = yaml_prefix + '.json'
    yaml_group_mark = '.'

    path_generated_dv3 = 'dv3'

    def run(self):
        js_glob_list = self.get_js_glob_list()
        groups = {}
        for glob_item in js_glob_list:
            if glob_item.endswith(self.yaml_suffix):
                basename = os.path.basename(glob_item)
                group = basename[: -len(self.yaml_suffix)].rsplit(self.yaml_group_mark, 1)[-1]
                array = groups.setdefault(group, [])
                real_glob_item = glob_item[
                                 :-(len(self.yaml_group_mark) + len(group) + len(self.yaml_suffix))] + self.yaml_prefix
                for filepath in self.list_glob_filepath(real_glob_item):
                    array.append(filepath)
        for group, paths in groups.items():
            res_loader = ResLoader.from_template_file(*(dict(filepath=filepath) for filepath in paths))
            for data in res_loader.data_list:
                data.pop('__context__', None)
                filepath = data.pop('__file__')
                index = filepath.rfind(self.path_generated_dv3)
                uid = hashlib.sha256(filepath.encode('utf-8')).hexdigest()
                filepath_new = self.get_dek_path_generated(
                    os.path.splitext(filepath[index:])[0] + '.' + uid + self.yaml_group_mark + group + self.yaml_suffix)
                write_file(filepath_new, s=json.dumps(data, indent=2, ensure_ascii=False, sort_keys=False))



import os
import json
from .base import PluginVue, PluginUniapp


class PluginVueConfig(PluginVue, PluginUniapp):
    vue_config_standard_name = 'vue.config.js'
    vue_config_marker = '//@marker-config-add'

    @property
    def transpile_dependencies(self):
        result = []

        for dir_dek in self.dek_dir_list:
            package_name = dir_dek[len(self.node_modules_dir) + 1:].replace('\\', '/')
            if package_name:
                result.append(package_name)

        for data_dek in self.dek_info_list:
            items = (data_dek.get(self.vue_config_standard_name) or {}).get('transpileDependencies') or []
            result.extend(items)

        return result

    def run(self):
        version = self.get_vue_version()
        if (not version or version[0] != 2) and not self.is_uniapp_project():
            return
        if not os.path.isfile(self.vue_config_standard_filepath):
            return
        s = self.load_text(self.vue_config_standard_filepath)
        str_list = []
        for item in sorted(set(self.transpile_dependencies)):
            mk = f'marker--addTranspileDependencies--{self.get_data_uid(item)}'
            if mk not in s:
                str_list.append(f'vueConfig.addTranspileDependencies({json.dumps(mk)}, {json.dumps(item)})')
        index = s.find(self.vue_config_marker)
        ss = s[:index] + '\n' + '\n'.join(str_list) + '\n' + s[index:]
        self.save_text(self.vue_config_standard_filepath, ss)

    @property
    def vue_config_standard_filepath(self):
        return os.path.join(self.project_dir, self.vue_config_standard_name)



import os
import sys
import hashlib
import glob
import string
import json
import json5
from collections import OrderedDict
from dektools.file import read_text
from dektools.dict import flat, assign, dict_merge
from dekgen.tmpl.template import Template


class PluginBase:
    package_standard_name = 'package.json'
    package_dek_name = 'package.dek.json'
    node_modules_dir_name = 'node_modules'
    dek_path_generated = 'dek'

    def __init__(self, project_dir):
        self.project_dir = os.path.normpath(project_dir)

    def get_pkg_version(self, name):
        info = self.load_package_standard()
        version = (info.get('dependencies') or {}).get(name) or (info.get('devDependencies') or {}).get(name) or ''
        version = "".join([x for x in version if x in string.digits or x == '.'])
        return [int(x) for x in version.split('.')] if version else []

    def load_package_standard(self, default=None):
        if os.path.exists(self.package_standard_filepath):
            return self.load_json(self.package_standard_filepath)
        else:
            return default or {}

    def get_dek_path_generated(self, *args):
        return os.path.normpath(os.path.join(self.project_dir, self.dek_path_generated, *args))

    @classmethod
    def get_package_json(cls, filepath):
        data = {}
        package_path = os.path.join(filepath, cls.package_standard_name)
        package_dek_path = os.path.join(filepath, cls.package_dek_name)
        if os.path.isfile(package_path):
            data = cls.load_json(package_path)
        elif os.path.isfile(package_dek_path):
            data = cls.load_json(package_dek_path)
        return data

    @property
    def package_standard_filepath(self):
        return os.path.join(self.project_dir, self.package_standard_name)

    @property
    def package_dek_filepath(self):
        return os.path.join(self.project_dir, self.package_dek_name)

    @property
    def node_modules_dir(self):
        return os.path.join(self.project_dir, self.node_modules_dir_name)

    @staticmethod
    def log(*args):
        for arg in args:
            sys.stdout.write(str(arg))
            sys.stdout.write(' ')
        sys.stdout.write('\n')
        sys.stdout.flush()

    @staticmethod
    def get_data_uid(data):
        s = json.dumps(data, sort_keys=True)
        h = hashlib.sha256()
        h.update(s.encode('utf-8'))
        return h.hexdigest()

    @staticmethod
    def load_text(filepath):
        with open(filepath, 'r', encoding='utf-8') as f:
            return f.read()

    @staticmethod
    def save_text(filepath, s):
        if os.path.exists(filepath):
            os.remove(filepath)
        dir_path = os.path.dirname(filepath)
        if not os.path.exists(dir_path):
            os.makedirs(dir_path)
        with open(filepath, 'w', encoding='utf-8') as f:
            f.write(s)

    @classmethod
    def load_json(cls, filepath, ser=json):
        return ser.loads(cls.load_text(filepath), object_pairs_hook=OrderedDict)

    @classmethod
    def save_json(cls, filepath, data):
        cls.save_text(filepath, json.dumps(data, indent=4))


class Plugin(PluginBase):
    template_cls = Template

    def __init__(self, project_dir, dek_info_list, dek_dir_list, dek_dev_dir_list, share_data):
        super().__init__(project_dir)
        self.dek_info_list = dek_info_list
        self.dek_dir_list = [os.path.normpath(p) for p in dek_dir_list]
        self.dek_dev_dir_list = [os.path.normpath(p) for p in dek_dev_dir_list]
        self.share_data = share_data

    def run(self):
        raise NotImplemented()

    @staticmethod
    def flat_variables(d):
        return flat(d, '__')

    def get_variables(self):
        data_package_standard = self.load_package_standard()
        variables = dict(
            package=dict(
                name=data_package_standard['name'],
                version=data_package_standard['version'],
                description=data_package_standard['description']
            )
        )
        for dek in self.dek_info_list:
            variables = assign(variables, dek.get('vars') or {})
        variables = assign(variables, self.share_data.get('variables') or {})
        return variables

    def merge_from_key(self, key):
        result = OrderedDict()
        for data in self.dek_info_list:
            env_set = data.get(key)
            if env_set:
                dict_merge(result, env_set)
        return result

    @property
    def dek_dir_list_for_scan(self):
        dir_list = [d for d in self.dek_dir_list if d != self.project_dir]
        for d in os.listdir(self.project_dir):
            fp = os.path.join(self.project_dir, d)
            if os.path.isdir(fp) and d not in ['dist', 'build', self.node_modules_dir_name, self.dek_path_generated]:
                dir_list.append(fp)
        return dir_list

    @property
    def template_full(self):
        return self.template_cls(self.flat_variables(self.get_variables()))

    def render_json(self, data):
        return json.loads(self.template_full.render(json.dumps(data)), object_pairs_hook=OrderedDict)

    def list_glob_filepath(self, glob_item):
        for dir_scan in self.dek_dir_list_for_scan:
            for filepath in glob.glob(dir_scan + glob_item, recursive=True):
                yield filepath

    def get_js_glob_list(self):
        import_meta_glob = 'import.meta.glob'
        glob_list = set()
        for filepath in self.list_glob_filepath('/**/*.js'):
            js = read_text(filepath)
            index = 0
            while index < len(js):
                index = js.find(import_meta_glob, index)
                if index != -1:
                    index += len(import_meta_glob)
                    begin = None
                    end = None
                    while index < len(js):
                        if js[index] in '"\'':
                            if begin is None:
                                begin = index
                            elif end is None:
                                end = index
                                break
                        index += 1
                    glob_list.add(js[begin + 1:end])
                else:
                    break
        return glob_list


class PluginVue(Plugin):
    def get_vue_version(self):
        return self.get_pkg_version('vue')


class PluginUniapp(Plugin):
    manifest_name = 'manifest.json'

    @property
    def manifest_filepath(self):
        return os.path.join(self.project_dir, self.manifest_name)

    def is_uniapp_project(self):
        return os.path.isfile(self.manifest_filepath)

    def get_uniapp_vue_version(self):
        data = self.load_json(self.manifest_filepath, json5)
        version = data.get('vueVersion')
        if version:
            return int(version)
        return None



import json
import base64
from dektools.file import read_text, write_file


class EnvVars:
    marker_begin = '<meta name="{key}"'
    marker_end = '\n'

    def __init__(self, path):
        self.path = path

    def update(self, key, value):
        text = read_text(self.path)
        marker_begin = self.marker_begin.format(key=key)
        marker_end = self.marker_end.format(key=key)
        index_begin = text.find(marker_begin)
        if index_begin == -1:
            raise ValueError(f"Can't find begin marker: {marker_begin}")
        index_end = text.find(marker_end, index_begin)
        if index_end == -1:
            raise ValueError(f"Can't find end marker: {marker_end}")
        if value:
            content = base64.b64encode(json.dumps(value).encode('utf-8')).decode('ascii')
        else:
            content = ""
        text_new = text[:index_begin] + f'{marker_begin} content="{content}">' + text[index_end:]
        write_file(self.path, s=text_new)



import os
import json
import typer
from typing import List
from typing_extensions import Annotated, Optional
from dynaconf.loaders.env_loader import load_from_env
from dektools.file import read_text
from dektools.typer import load_yaml_files
from dektools.dict import dict_merge
from ..core.base.env_vars import EnvVars
from ..core import run_plugin_dv3_env_vars, PluginDv3EnvVars

app = typer.Typer(add_completion=False)


@app.command()
def to(env, path=None, files: Annotated[Optional[List[str]], typer.Option('--file')] = None):  # env: dev, stage, prod
    run_plugin_dv3_env_vars()
    path_project = path or os.getcwd()
    data = json.loads(read_text(os.path.join(path_project, PluginDv3EnvVars.env_path_data)))
    dict_merge(data, load_yaml_files(files))
    ev = EnvVars(os.path.join(path_project, PluginDv3EnvVars.env_path_src))
    ev.update('envBase', data.get('base'))
    ev.update('envEnv', data.get(env))


@app.command()
def final(
        path=None,
        prefix=f'{__name__.partition(".")[0]}_ENV',
        files: Annotated[Optional[List[str]], typer.Option('--file')] = None
):
    path_project = path or os.getcwd()
    data = load_yaml_files(files)
    load_from_env(key=None, prefix=prefix, obj=data, silent=True)
    data = {k: v for k, v in data.items() if k.isupper()}
    EnvVars(os.path.join(path_project, PluginDv3EnvVars.env_path_src)).update('env', data)



import os
import fitz
from svglib.svglib import svg2rlg
from reportlab.graphics import renderPDF
from .base import Plugin


def write_ico(input_img, output_img, size, dpi):
    drawing = svg2rlg(input_img)
    pdf = renderPDF.drawToString(drawing)
    doc = fitz.Document(stream=pdf)
    pix = doc.load_page(0).get_pixmap(alpha=True, dpi=dpi)
    dir_target = os.path.dirname(output_img)
    if not os.path.isdir(dir_target):
        os.makedirs(dir_target)
    pix.pil_save(output_img, sizes=[(size, size)])


class PluginFaviconIco(Plugin):
    favicon_ico_size = 128
    favicon_ico_dpi = 300

    def run(self):
        if not os.path.isfile(self.favicon_svg_filepath):
            return
        write_ico(self.favicon_svg_filepath, self.favicon_ico_filepath, self.favicon_ico_size, self.favicon_ico_dpi)

    @property
    def favicon_svg_filepath(self):
        return os.path.join(self.project_dir, 'res', 'favicon.svg')

    @property
    def favicon_ico_filepath(self):
        return os.path.join(self.project_dir, 'public', 'favicon.ico')



import typer
from typing_extensions import Annotated
from dektools.shell import shell_wrapper
from dektools.url import split_auth_url
from dektools.output import print_data_or_value
from ..core import execute_install, execute_build, execute_package_sure, execute_server, run_plugin_dv3_yaml, \
    clear_pkg_cache, plugin_base
from ..gen.tmpl import ShellGenerator

app = typer.Typer(add_completion=False)


@app.command()
def install():
    execute_install()


@app.command()
def build():
    execute_build()


@app.command()
def sure(force: Annotated[bool, typer.Option("--force/--no-force")] = False):
    execute_package_sure(force)


@app.command()
def server():
    execute_server(False)


@app.command()
def serverfull():
    execute_server(True)


@app.command()
def clearcache():
    clear_pkg_cache()


@app.command()
def yaml():
    run_plugin_dv3_yaml()


@app.command()
def shell(path: Annotated[str, typer.Argument()] = "."):
    ShellGenerator(path, None).render()


@app.command()
def login(url):
    url, scope, token = split_auth_url(url)
    shell_wrapper(f"npm config set {scope}:registry {url}")
    shell_wrapper(f"""npm config set -- '{url.split(":", 1)[-1]}:_authToken' "{token}" """)


@app.command()
def logout(url):
    url, scope, _ = split_auth_url(url)
    shell_wrapper(f"npm config delete {scope}:registry")
    shell_wrapper(f'npm config delete {url.split(":", 1)[-1]}:_authToken')


@app.command()
def meta(path, expression: Annotated[str, typer.Argument()] = ""):
    print_data_or_value(plugin_base.get_package_json(path), expression)



import os
import shutil
from .base import Plugin


class PluginDv3(Plugin):
    path_generated_dv3 = 'dv3'

    def run(self):
        path_root = self.get_dek_path_generated(self.path_generated_dv3)
        if os.path.exists(path_root):
            shutil.rmtree(path_root)



from .plugins import PluginUniappVars, PluginPackageJson, PluginBabelConfig, PluginVueConfig, PluginOverrideRoot, \
    PluginDv3, PluginFaviconIco, PluginViteConfig, PluginDv3Svg, PluginDv3Yaml, PluginPackages, PluginRequests, \
    PluginDv3Server, PluginDv3EnvVars

g_plugin_list = [
    PluginUniappVars,
    PluginOverrideRoot,
    PluginPackageJson,
    PluginBabelConfig,
    PluginVueConfig,
    PluginViteConfig,
    PluginFaviconIco,
    PluginDv3,
    PluginDv3Svg,
    PluginDv3Yaml,

    PluginPackages,
    PluginDv3Server,
    PluginRequests,
    PluginDv3EnvVars
]


def run_plugins(project_dir, dek_info_list, dek_dir_list, dek_dev_dir_list, plugin_list=None):
    share_data = {}
    for plugin_cls in plugin_list or g_plugin_list:
        plugin = plugin_cls(project_dir, dek_info_list, dek_dir_list, dek_dev_dir_list, share_data)
        plugin.run()



import os
import json
from .base import Plugin


class PluginBabelConfig(Plugin):
    babel_config_standard_name = 'babel.config.js'
    babel_config_marker = '//@marker-config-add'

    def run(self):
        if not os.path.isfile(self.babel_config_standard_filepath):
            return
        for data in self.dek_info_list:
            config_info = data.get(self.babel_config_standard_name) or {}
            if config_info:
                config_info = self.render_json(config_info)
                s = self.load_text(self.babel_config_standard_filepath)
                str_list = []
                for k, v in config_info.get('presets', {}).items():
                    mk = f'marker-preset---{k}'
                    if mk not in s:
                        str_list.append(f'babelConfig.addPreset({json.dumps(mk)}, {json.dumps(v)})')
                for k, v in config_info.get('plugins', {}).items():
                    mk = f'marker-plugin---{k}'
                    if mk not in s:
                        str_list.append(f'babelConfig.addPlugin({json.dumps(mk)}, {json.dumps(v)})')
                index = s.find(self.babel_config_marker)
                ss = s[:index] + '\n' + '\n'.join(str_list) + '\n' + s[index:]
                self.save_text(self.babel_config_standard_filepath, ss)

    @property
    def babel_config_standard_filepath(self):
        return os.path.join(self.project_dir, self.babel_config_standard_name)



import functools
from pathlib import Path
from dektools.dict import sorted_dict
from dekgen.code.python.generator import CodeGenerator
from dektools.variables import get_user_email_from_git


class GeneratorBasic(CodeGenerator):
    TEMPLATE_DIR = Path(__file__).resolve().parent / 'templatefiles'

    def variables_data(self):
        return self.instance or {}


class ShellGenerator(GeneratorBasic):
    template_name = 'shell'



import os
import glob
from dektools.file.operation import copy_path
from .base import Plugin


class PluginDv3Server(Plugin):
    path_generated_servers = 'dv3/servers'
    glob_server_yaml = '/**/dv3/servers/**/*.yaml'
    glob_prefix_path = '/dv3/servers/'.replace('/', os.sep).replace('\\', os.sep)

    def run(self):
        if not self.share_data.get('packages'):
            return
        packages = {v: k for k, v in self.share_data['packages'].items()}
        packages_dir = sorted(packages.keys(), reverse=True, key=lambda x: len(x))
        for dir_scan in self.dek_dir_list_for_scan:
            server_yaml_dir = None
            package_name = None
            for filepath in glob.glob(dir_scan + self.glob_server_yaml, recursive=True):
                filepath = os.path.normpath(filepath)
                for p in packages_dir:
                    if filepath.startswith(p):
                        package_name = packages[p]
                        break
                if package_name is None:
                    raise Exception(f'filepath not in {packages_dir}')
                index_sep = filepath.rfind(self.glob_prefix_path) + len(self.glob_prefix_path) - 1
                server_yaml_dir = filepath[:index_sep]
                break
            if server_yaml_dir:
                copy_path(server_yaml_dir, self.get_dek_path_generated(self.path_generated_servers, package_name))



import os
from .base import Plugin


class PluginRequests(Plugin):
    path_generated_icons = 'dv3/requests'

    def run(self):
        self.walk(self.dir_server)

    @property
    def dir_server(self):
        return os.path.join(self.project_dir, 'server')

    def walk(self, root):
        if os.path.isdir(root):
            for name in os.listdir(root):
                rn = os.path.join(root, name)
                if os.path.isdir(rn) and name not in ['.venv']:
                    self.walk(rn)
                elif name == 'requests.djcreator.json':
                    if 'dumppredefined__' not in root.split(os.sep):
                        name_root = rn[len(self.dir_server) + 1:].split(os.sep, 1)[0]
                        self.save_json(
                            self.get_dek_path_generated(self.path_generated_icons, f'{name_root}.json'),
                            self.load_json(rn))



import os
import json
from dektools.file import write_file
from .base import Plugin
from ..base.env_vars import EnvVars


class PluginDv3EnvVars(Plugin):
    env_key = 'vars.env'
    env_path_data = 'deknp.env.log'
    env_path_src = 'index.html'

    def run(self):
        result = self.merge_from_key(self.env_key)
        write_file(
            os.path.join(self.project_dir, self.env_path_data),
            json.dumps(result, indent=2, ensure_ascii=False, sort_keys=True)
        )
        path_src = os.path.join(self.project_dir, self.env_path_src)
        if os.path.isfile(path_src):
            ev = EnvVars(path_src)
            ev.update('envBase', result.get('base'))
            ev.update('envEnv', result.get('dev'))



import os
from .base import PluginUniapp
from dektools.dict import assign


# def get_easy_com_dict(prefix, names):
#     if not names:
#         return {}
#     name = names[0]
#     if len(names) == 1:
#         return {name: f'{prefix}/{name}/{name}.vue'}
#     cursor = 0
#     while True:
#         cursor_char = name[cursor:cursor + 1]
#         for x in names:
#             xx = x[cursor:cursor + 1]
#             if xx != cursor_char or '' in [xx, cursor_char]:
#                 start = name[:cursor]
#                 if not start:
#                     raise Exception(f'empty error for: {prefix} {names}')
#                 return {f'^{start}(.*)': f'{prefix}/{start}$1/{start}$1.vue'}
#         cursor += 1

def get_easy_com_dict(prefix, names):
    return {f'^{name}$': f'{prefix}/{name}/{name}.vue' for name in names}


class PluginUniappVars(PluginUniapp):
    def variables__easy_com(self):
        def pkg_args(fn, base):
            return [
                fn[len(base) + 1:].replace('\\', '/'),
                [f for f in os.listdir(fn) if os.path.isdir(os.path.join(fn, f))]
            ]

        variables = {}
        for root, dirs, _ in os.walk(self.node_modules_dir):
            for name in dirs:
                rn = os.path.join(root, name)
                if name == 'components' and os.path.basename(os.path.dirname(os.path.dirname(rn))) == 'uni_modules':
                    variables.update(get_easy_com_dict(*pkg_args(rn, self.node_modules_dir)))

        # dir_components = os.path.join(self.project_dir, 'components')
        # if os.path.exists(dir_components):
        #     variables.update(get_easy_com_dict(*pkg_args(dir_components, self.project_dir)))

        return {
            'uniapp': {
                'easycom': variables or '{}'
            }
        }

    def run(self):
        self.share_data['variables'] = assign(
            self.share_data.get('variables') or {},
            # self.variables__easy_com()
        )



import os
import shutil
import json
import sys
from collections import OrderedDict, ChainMap
from copy import deepcopy
from dektools.file import write_file, read_text, remove_path
from dektools.yaml import yaml
from dektools.shell import Cli, shell_wrapper
from dektools.dict import dict_merge
from .plugins.base import PluginBase
from .run import run_plugins, PluginDv3Yaml, PluginDv3EnvVars

project_dir = os.getcwd()
plugin_base = PluginBase(project_dir)

path_project_log = os.path.join(project_dir, 'deknp.log')
path_project_tree_log = os.path.join(project_dir, 'deknp.tree.log')

cli = Cli('pnpm', 'yarn', 'npm').cur


def run_install_package_cmd(data):
    for k, v in (data.get('dependencies') or {}).items():
        shell_wrapper(f'{cli} add "{k}@{v}"', True)
    for k, v in (data.get('devDependencies') or {}).items():
        shell_wrapper(f'{cli} add -D "{k}@{v}"', True)


def filter_packages(data):
    def pop(d):
        for k in list(d):
            if os.path.exists(os.path.join(plugin_base.node_modules_dir, k)):
                d.pop(k)

    data = deepcopy(data)
    pop(data.get('dependencies') or {})
    pop(data.get('devDependencies') or {})
    return data


def merge_package(a, b):
    result = deepcopy(a)
    result.update([
        ('dependencies',
         OrderedDict(ChainMap(a.get('dependencies', OrderedDict()), b.get('dependencies', OrderedDict())))),
        ('devDependencies',
         OrderedDict(ChainMap(a.get('devDependencies', OrderedDict()), b.get('devDependencies', OrderedDict())))),
    ])
    return result


def execute_build():
    if cli == 'yarn':
        shell_wrapper(f'{cli} build')
    else:
        shell_wrapper(f'{cli} run build')


def execute_install():
    def walk(dir_dek, dev=False):
        if dir_dek in handed_set:
            return
        handed_set.add(dir_dek)
        fp = os.path.join(dir_dek, plugin_base.package_dek_name)
        if os.path.exists(fp):
            data_dek = plugin_base.load_json(fp)
            if dir_dek == project_dir:
                data_dek_run = data_dek.get('dek', {}).get('run') or {}
                if data_dek_run:
                    dict_merge(data_dek, data_dek_run)
            data_dek_list.append(data_dek)
            data = merge_package(plugin_base.load_package_standard(deepcopy(data_dek)), data_dek)
            plugin_base.save_json(plugin_base.package_standard_filepath, data)
            run_install_package_cmd(filter_packages(data))
            for k in reversed(data.get('dependencies', OrderedDict())):
                walk(os.path.join(plugin_base.node_modules_dir, k))
            for k in reversed(data.get('devDependencies', OrderedDict())):
                walk(os.path.join(plugin_base.node_modules_dir, k), True)

            dek_dir_list.append(dir_dek)
            if dev:
                dek_dev_dir_list.append(dir_dek)
            dek_info = data_dek.get('dek')
            if dek_info:
                dek_info_list.append(dek_info)

    handed_set = set()
    dek_dir_list = []
    dek_dev_dir_list = []
    dek_info_list = []
    data_dek_list = []
    if os.path.exists(plugin_base.package_dek_filepath):
        dd = plugin_base.load_json(plugin_base.package_dek_filepath)
        dd.update([
            ('dependencies', {}),
            ('devDependencies', {}),
        ])
        plugin_base.save_json(plugin_base.package_standard_filepath, dd)
    walk(project_dir)
    write_file(path_project_tree_log, yaml.dumps(data_dek_list))
    write_file(path_project_log,
               json.dumps([dek_info_list, dek_dir_list, dek_dev_dir_list], indent=2, ensure_ascii=False))
    run_plugins(project_dir, dek_info_list, dek_dir_list, dek_dev_dir_list)
    return 0


def execute_package_sure(force=False):
    if os.path.exists(plugin_base.package_dek_filepath) and \
            (force or not os.path.exists(plugin_base.package_standard_filepath)):
        data_dek = plugin_base.load_json(plugin_base.package_dek_filepath)
        data_dek.pop('dek', None)
        plugin_base.save_json(plugin_base.package_standard_filepath, data_dek)


def execute_server(begin=False):
    dir_servers = os.path.join(project_dir, 'dek', 'dv3', 'servers')
    project_name = plugin_base.get_package_json(
        project_dir
    )['name'].split('/')[-1].replace('-', '').replace('_', '').lower()
    if os.path.isdir(dir_servers) and os.listdir(dir_servers):
        dir_work = os.path.join(project_dir, 'server')
        if not os.path.isdir(dir_work):
            os.makedirs(dir_work)
        dir_material = os.path.join(dir_work, '.material')
        if not os.path.exists(dir_material):
            shell_wrapper(f'djcreator wrapper clean --project={project_name}')
            shell_wrapper(f'djcreator project {project_name} {dir_work}')
        dir_project = os.path.join(dir_material, f'{project_name}.yaml')
        if os.path.isdir(dir_project):
            shutil.rmtree(dir_project)
        shutil.copytree(dir_servers, dir_project)
        shell_wrapper(f'dekshell rf .shell.initshell.pysh', chdir=dir_work)
        if begin:
            shell_wrapper(f'dekshell rf .shell.generate.{project_name}.pysh', chdir=dir_material)
            shell_wrapper(f'deknp install', chdir=project_dir)


def run_plugin_dv3_yaml():
    run_plugins(project_dir, *json.loads(read_text(path_project_log)), [PluginDv3Yaml])


def run_plugin_dv3_env_vars():
    run_plugins(project_dir, *json.loads(read_text(path_project_log)), [PluginDv3EnvVars])


def get_pnpm_cache_dir():  # https://github.com/pnpm/pnpm/blob/v8.10.5/config/config/src/dirs.ts
    if os.getenv('XDG_CACHE_HOME'):
        return os.path.join(os.getenv('XDG_CACHE_HOME'), 'pnpm')
    if sys.platform == 'darwin':
        return os.path.join(os.path.expanduser('~'), 'Library/Caches/pnpm')
    if sys.platform != 'win32':
        return os.path.join(os.path.expanduser('~'), '.cache/pnpm')
    if os.getenv('LOCALAPPDATA'):
        return os.path.join(os.getenv('LOCALAPPDATA'), 'pnpm-cache')
    return os.path.join(os.path.expanduser('~'), '.pnpm-cache')


def clear_pkg_cache():
    pkg_info = plugin_base.get_package_json(project_dir)
    name, version = pkg_info['name'], pkg_info['version']
    path_meta = os.path.join(get_pnpm_cache_dir(), 'metadata')
    if os.path.isdir(path_meta):
        for host in os.listdir(path_meta):
            path_pkg = os.path.join(path_meta, host, name + '.json')
            if os.path.isfile(path_pkg):
                data = json.loads(read_text(path_pkg))
                versions = data.get('versions') or {}
                exists = versions.pop(version, None)
                if exists is not None:
                    if versions:
                        write_file(path_pkg, json.dumps(data))
                    else:
                        remove_path(path_pkg)
